@@ -1,17 +1,13 @@
-//! Trait implementations for `BitVec`.
+//! General trait implementations for bit-vectors.
 
-use crate::{
-	boxed::BitBox,
-	order::BitOrder,
-	slice::BitSlice,
-	store::BitStore,
-	vec::BitVec,
+use alloc::{
+	borrow::{
+		Cow,
+		ToOwned,
+	},
+	vec::Vec,
 };
-
-use alloc::vec::Vec;
-
 use core::{
-	any,
 	borrow::{
 		Borrow,
 		BorrowMut,
@@ -20,400 +16,368 @@ use core::{
 	convert::TryFrom,
 	fmt::{
 		self,
-		Binary,
 		Debug,
 		Display,
 		Formatter,
-		LowerHex,
-		Octal,
-		Pointer,
-		UpperHex,
 	},
 	hash::{
 		Hash,
 		Hasher,
 	},
+	marker::Unpin,
 };
 
-use tap::{
-	pipe::Pipe,
-	tap::Tap,
+use super::BitVec;
+use crate::{
+	array::BitArray,
+	boxed::BitBox,
+	order::BitOrder,
+	slice::BitSlice,
+	store::BitStore,
+	view::BitViewSized,
 };
 
 #[cfg(not(tarpaulin_include))]
-impl<O, T> Borrow<BitSlice<O, T>> for BitVec<O, T>
+impl<T, O> Borrow<BitSlice<T, O>> for BitVec<T, O>
 where
-	O: BitOrder,
 	T: BitStore,
+	O: BitOrder,
 {
-	#[inline(always)]
-	fn borrow(&self) -> &BitSlice<O, T> {
+	fn borrow(&self) -> &BitSlice<T, O> {
 		self.as_bitslice()
 	}
 }
 
 #[cfg(not(tarpaulin_include))]
-impl<O, T> BorrowMut<BitSlice<O, T>> for BitVec<O, T>
+impl<T, O> BorrowMut<BitSlice<T, O>> for BitVec<T, O>
 where
-	O: BitOrder,
 	T: BitStore,
+	O: BitOrder,
 {
-	#[inline(always)]
-	fn borrow_mut(&mut self) -> &mut BitSlice<O, T> {
+	fn borrow_mut(&mut self) -> &mut BitSlice<T, O> {
 		self.as_mut_bitslice()
 	}
 }
 
-impl<O, T> Clone for BitVec<O, T>
+#[cfg(not(tarpaulin_include))]
+impl<T, O> Clone for BitVec<T, O>
 where
-	O: BitOrder,
 	T: BitStore,
+	O: BitOrder,
 {
-	#[inline]
 	fn clone(&self) -> Self {
-		self.as_bitslice().pipe(Self::from_bitslice)
-	}
-
-	#[inline]
-	fn clone_from(&mut self, other: &Self) {
-		self.clear();
-		self.reserve(other.len());
-		self.with_vec(|v| {
-			v.extend(other.as_slice().iter().map(BitStore::load_value))
-		});
-		unsafe {
-			self.set_len(other.len());
-		}
-		self.pointer = self
-			.bitptr()
-			.tap_mut(|bp| unsafe { bp.set_head(other.bitptr().head()) })
-			.to_nonnull();
+		Self::from_bitslice(self.as_bitslice())
 	}
 }
 
-#[cfg(not(tarpaulin_include))]
-impl<O, T> Eq for BitVec<O, T>
+impl<T, O> Eq for BitVec<T, O>
 where
-	O: BitOrder,
 	T: BitStore,
+	O: BitOrder,
 {
 }
 
 #[cfg(not(tarpaulin_include))]
-impl<O, T> Ord for BitVec<O, T>
+impl<T, O> Ord for BitVec<T, O>
 where
-	O: BitOrder,
 	T: BitStore,
+	O: BitOrder,
 {
-	#[inline]
 	fn cmp(&self, other: &Self) -> cmp::Ordering {
 		self.as_bitslice().cmp(other.as_bitslice())
 	}
 }
 
 #[cfg(not(tarpaulin_include))]
-impl<O1, O2, T1, T2> PartialEq<BitVec<O2, T2>> for BitSlice<O1, T1>
+impl<T1, T2, O1, O2> PartialEq<BitVec<T2, O2>> for BitSlice<T1, O1>
 where
-	O1: BitOrder,
-	O2: BitOrder,
 	T1: BitStore,
 	T2: BitStore,
+	O1: BitOrder,
+	O2: BitOrder,
 {
-	#[inline]
-	fn eq(&self, other: &BitVec<O2, T2>) -> bool {
+	fn eq(&self, other: &BitVec<T2, O2>) -> bool {
 		self == other.as_bitslice()
 	}
 }
 
 #[cfg(not(tarpaulin_include))]
-impl<O1, O2, T1, T2> PartialEq<BitVec<O2, T2>> for &BitSlice<O1, T1>
+impl<T1, T2, O1, O2> PartialEq<BitVec<T2, O2>> for &BitSlice<T1, O1>
 where
-	O1: BitOrder,
-	O2: BitOrder,
 	T1: BitStore,
 	T2: BitStore,
+	O1: BitOrder,
+	O2: BitOrder,
 {
-	#[inline]
-	fn eq(&self, other: &BitVec<O2, T2>) -> bool {
+	fn eq(&self, other: &BitVec<T2, O2>) -> bool {
 		*self == other.as_bitslice()
 	}
 }
 
 #[cfg(not(tarpaulin_include))]
-impl<O1, O2, T1, T2> PartialEq<BitVec<O2, T2>> for &mut BitSlice<O1, T1>
+impl<T1, T2, O1, O2> PartialEq<BitVec<T2, O2>> for &mut BitSlice<T1, O1>
 where
-	O1: BitOrder,
-	O2: BitOrder,
 	T1: BitStore,
 	T2: BitStore,
+	O1: BitOrder,
+	O2: BitOrder,
 {
-	#[inline]
-	fn eq(&self, other: &BitVec<O2, T2>) -> bool {
+	fn eq(&self, other: &BitVec<T2, O2>) -> bool {
 		**self == other.as_bitslice()
 	}
 }
 
 #[cfg(not(tarpaulin_include))]
-impl<O, T, Rhs> PartialEq<Rhs> for BitVec<O, T>
+impl<T, O, Rhs> PartialEq<Rhs> for BitVec<T, O>
 where
-	O: BitOrder,
 	T: BitStore,
-	Rhs: ?Sized + PartialEq<BitSlice<O, T>>,
+	O: BitOrder,
+	Rhs: ?Sized + PartialEq<BitSlice<T, O>>,
 {
-	#[inline]
 	fn eq(&self, other: &Rhs) -> bool {
 		other == self.as_bitslice()
 	}
 }
 
 #[cfg(not(tarpaulin_include))]
-impl<O, T> PartialOrd<BitVec<O, T>> for BitSlice<O, T>
+impl<T1, T2, O1, O2> PartialOrd<BitVec<T2, O2>> for BitSlice<T1, O1>
 where
-	O: BitOrder,
-	T: BitStore,
+	T1: BitStore,
+	T2: BitStore,
+	O1: BitOrder,
+	O2: BitOrder,
 {
-	#[inline]
-	fn partial_cmp(&self, other: &BitVec<O, T>) -> Option<cmp::Ordering> {
+	fn partial_cmp(&self, other: &BitVec<T2, O2>) -> Option<cmp::Ordering> {
 		self.partial_cmp(other.as_bitslice())
 	}
 }
 
 #[cfg(not(tarpaulin_include))]
-impl<O, T, Rhs> PartialOrd<Rhs> for BitVec<O, T>
+impl<'a, T1, T2, O1, O2> PartialOrd<BitVec<T2, O2>> for &'a BitSlice<T1, O1>
 where
+	T1: BitStore,
+	T2: BitStore,
+	O1: BitOrder,
+	O2: BitOrder,
+{
+	fn partial_cmp(&self, other: &BitVec<T2, O2>) -> Option<cmp::Ordering> {
+		self.partial_cmp(other.as_bitslice())
+	}
+}
+
+#[cfg(not(tarpaulin_include))]
+impl<'a, T1, T2, O1, O2> PartialOrd<BitVec<T2, O2>> for &'a mut BitSlice<T1, O1>
+where
+	T1: BitStore,
+	T2: BitStore,
+	O1: BitOrder,
+	O2: BitOrder,
+{
+	fn partial_cmp(&self, other: &BitVec<T2, O2>) -> Option<cmp::Ordering> {
+		self.partial_cmp(other.as_bitslice())
+	}
+}
+
+#[cfg(not(tarpaulin_include))]
+impl<T, O, Rhs> PartialOrd<Rhs> for BitVec<T, O>
+where
+	T: BitStore,
 	O: BitOrder,
-	T: BitStore,
-	Rhs: ?Sized + PartialOrd<BitSlice<O, T>>,
+	Rhs: ?Sized + PartialOrd<BitSlice<T, O>>,
 {
-	#[inline]
 	fn partial_cmp(&self, other: &Rhs) -> Option<cmp::Ordering> {
 		other.partial_cmp(self.as_bitslice())
 	}
 }
 
 #[cfg(not(tarpaulin_include))]
-impl<O, T> AsRef<BitSlice<O, T>> for BitVec<O, T>
+impl<T, O> AsRef<BitSlice<T, O>> for BitVec<T, O>
 where
-	O: BitOrder,
 	T: BitStore,
+	O: BitOrder,
 {
-	#[inline(always)]
-	fn as_ref(&self) -> &BitSlice<O, T> {
+	fn as_ref(&self) -> &BitSlice<T, O> {
 		self.as_bitslice()
 	}
 }
 
 #[cfg(not(tarpaulin_include))]
-impl<O, T> AsMut<BitSlice<O, T>> for BitVec<O, T>
+impl<T, O> AsMut<BitSlice<T, O>> for BitVec<T, O>
 where
-	O: BitOrder,
 	T: BitStore,
+	O: BitOrder,
 {
-	#[inline(always)]
-	fn as_mut(&mut self) -> &mut BitSlice<O, T> {
+	fn as_mut(&mut self) -> &mut BitSlice<T, O> {
 		self.as_mut_bitslice()
 	}
 }
 
 #[cfg(not(tarpaulin_include))]
-impl<'a, O, T> From<&'a BitSlice<O, T>> for BitVec<O, T>
+impl<T, O> AsRef<BitVec<T, O>> for BitVec<T, O>
 where
-	O: BitOrder,
 	T: BitStore,
+	O: BitOrder,
 {
-	#[inline(always)]
-	fn from(slice: &'a BitSlice<O, T>) -> Self {
-		slice.to_bitvec()
+	fn as_ref(&self) -> &Self {
+		self
 	}
 }
 
 #[cfg(not(tarpaulin_include))]
-impl<'a, O, T> From<&'a mut BitSlice<O, T>> for BitVec<O, T>
+impl<T, O> AsMut<BitVec<T, O>> for BitVec<T, O>
 where
-	O: BitOrder,
 	T: BitStore,
+	O: BitOrder,
 {
-	#[inline(always)]
-	fn from(slice: &'a mut BitSlice<O, T>) -> Self {
-		slice.to_bitvec()
+	fn as_mut(&mut self) -> &mut Self {
+		self
 	}
 }
 
 #[cfg(not(tarpaulin_include))]
-impl<O, T> From<BitBox<O, T>> for BitVec<O, T>
+impl<T, O> From<&'_ BitSlice<T, O>> for BitVec<T, O>
+where
+	T: BitStore,
+	O: BitOrder,
+{
+	fn from(slice: &BitSlice<T, O>) -> Self {
+		Self::from_bitslice(slice)
+	}
+}
+
+#[cfg(not(tarpaulin_include))]
+impl<T, O> From<&'_ mut BitSlice<T, O>> for BitVec<T, O>
+where
+	T: BitStore,
+	O: BitOrder,
+{
+	fn from(slice: &mut BitSlice<T, O>) -> Self {
+		Self::from_bitslice(slice)
+	}
+}
+
+#[cfg(not(tarpaulin_include))]
+impl<A, O> From<BitArray<A, O>> for BitVec<A::Store, O>
 where
 	O: BitOrder,
-	T: BitStore,
+	A: BitViewSized,
 {
-	#[inline(always)]
-	fn from(boxed: BitBox<O, T>) -> Self {
+	fn from(array: BitArray<A, O>) -> Self {
+		array.as_bitslice().to_owned()
+	}
+}
+
+#[cfg(not(tarpaulin_include))]
+impl<T, O> From<BitBox<T, O>> for BitVec<T, O>
+where
+	T: BitStore,
+	O: BitOrder,
+{
+	fn from(boxed: BitBox<T, O>) -> Self {
 		boxed.into_bitvec()
 	}
 }
 
 #[cfg(not(tarpaulin_include))]
-impl<O, T> Into<Vec<T>> for BitVec<O, T>
+impl<T, O> From<BitVec<T, O>> for Vec<T>
 where
-	O: BitOrder,
 	T: BitStore,
+	O: BitOrder,
 {
-	#[inline(always)]
-	fn into(self) -> Vec<T> {
-		self.into_vec()
+	fn from(bv: BitVec<T, O>) -> Self {
+		bv.into_vec()
 	}
 }
 
 #[cfg(not(tarpaulin_include))]
-impl<O, T> TryFrom<Vec<T>> for BitVec<O, T>
+impl<'a, T, O> From<Cow<'a, BitSlice<T, O>>> for BitVec<T, O>
 where
 	O: BitOrder,
+	T: 'a + BitStore,
+{
+	fn from(cow: Cow<'a, BitSlice<T, O>>) -> Self {
+		cow.into_owned()
+	}
+}
+
+#[cfg(not(tarpaulin_include))]
+impl<T, O> TryFrom<Vec<T>> for BitVec<T, O>
+where
 	T: BitStore,
+	O: BitOrder,
 {
 	type Error = Vec<T>;
 
-	#[inline(always)]
 	fn try_from(vec: Vec<T>) -> Result<Self, Self::Error> {
 		Self::try_from_vec(vec)
 	}
 }
 
 #[cfg(not(tarpaulin_include))]
-impl<O, T> Default for BitVec<O, T>
+impl<T, O> Default for BitVec<T, O>
 where
-	O: BitOrder,
 	T: BitStore,
+	O: BitOrder,
 {
-	#[inline(always)]
 	fn default() -> Self {
 		Self::new()
 	}
 }
 
-#[cfg(not(tarpaulin_include))]
-impl<O, T> Debug for BitVec<O, T>
+impl<T, O> Debug for BitVec<T, O>
 where
-	O: BitOrder,
 	T: BitStore,
+	O: BitOrder,
 {
-	#[inline]
 	fn fmt(&self, fmt: &mut Formatter) -> fmt::Result {
-		if fmt.alternate() {
-			Pointer::fmt(self, fmt)?;
-			fmt.write_str(" ")?;
-		}
-		Display::fmt(self.as_bitslice(), fmt)
+		self.as_bitspan().render(fmt, "Vec", &[(
+			"capacity",
+			&self.capacity() as &dyn Debug,
+		)])?;
+		fmt.write_str(" ")?;
+		Display::fmt(self, fmt)
 	}
 }
 
-#[cfg(not(tarpaulin_include))]
-impl<O, T> Display for BitVec<O, T>
-where
-	O: BitOrder,
-	T: BitStore,
-{
-	#[inline]
-	fn fmt(&self, fmt: &mut Formatter) -> fmt::Result {
-		Display::fmt(self.as_bitslice(), fmt)
-	}
+easy_fmt! {
+	impl Binary
+	impl Display
+	impl LowerHex
+	impl Octal
+	impl Pointer
+	impl UpperHex
+	for BitVec
 }
 
 #[cfg(not(tarpaulin_include))]
-impl<O, T> Binary for BitVec<O, T>
+impl<T, O> Hash for BitVec<T, O>
 where
-	O: BitOrder,
 	T: BitStore,
-{
-	#[inline]
-	fn fmt(&self, fmt: &mut Formatter) -> fmt::Result {
-		Binary::fmt(self.as_bitslice(), fmt)
-	}
-}
-
-#[cfg(not(tarpaulin_include))]
-impl<O, T> LowerHex for BitVec<O, T>
-where
 	O: BitOrder,
-	T: BitStore,
 {
-	#[inline]
-	fn fmt(&self, fmt: &mut Formatter) -> fmt::Result {
-		LowerHex::fmt(self.as_bitslice(), fmt)
-	}
-}
-
-#[cfg(not(tarpaulin_include))]
-impl<O, T> Octal for BitVec<O, T>
-where
-	O: BitOrder,
-	T: BitStore,
-{
-	#[inline]
-	fn fmt(&self, fmt: &mut Formatter) -> fmt::Result {
-		Octal::fmt(self.as_bitslice(), fmt)
-	}
-}
-
-#[cfg(not(tarpaulin_include))]
-impl<O, T> Pointer for BitVec<O, T>
-where
-	O: BitOrder,
-	T: BitStore,
-{
-	#[inline]
-	fn fmt(&self, fmt: &mut Formatter) -> fmt::Result {
-		self.bitptr()
-			.render(fmt, "Vec", Some(any::type_name::<O>()), &[(
-				"capacity",
-				&self.capacity() as &dyn Debug,
-			)])
-	}
-}
-
-#[cfg(not(tarpaulin_include))]
-impl<O, T> UpperHex for BitVec<O, T>
-where
-	O: BitOrder,
-	T: BitStore,
-{
-	#[inline]
-	fn fmt(&self, fmt: &mut Formatter) -> fmt::Result {
-		UpperHex::fmt(self.as_bitslice(), fmt)
-	}
-}
-
-#[cfg(not(tarpaulin_include))]
-impl<O, T> Hash for BitVec<O, T>
-where
-	O: BitOrder,
-	T: BitStore,
-{
-	#[inline]
 	fn hash<H>(&self, state: &mut H)
 	where H: Hasher {
 		self.as_bitslice().hash(state)
 	}
 }
 
-#[cfg(not(tarpaulin_include))]
-unsafe impl<O, T> Send for BitVec<O, T>
+unsafe impl<T, O> Send for BitVec<T, O>
 where
-	O: BitOrder,
 	T: BitStore,
+	O: BitOrder,
 {
 }
 
-#[cfg(not(tarpaulin_include))]
-unsafe impl<O, T> Sync for BitVec<O, T>
+unsafe impl<T, O> Sync for BitVec<T, O>
 where
-	O: BitOrder,
 	T: BitStore,
+	O: BitOrder,
 {
 }
 
-#[cfg(not(tarpaulin_include))]
-impl<O, T> Unpin for BitVec<O, T>
+impl<T, O> Unpin for BitVec<T, O>
 where
-	O: BitOrder,
 	T: BitStore,
+	O: BitOrder,
 {
 }
